/*
 * Copyright 2005-2013 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.springframework.ldap.control;

import com.sun.jndi.ldap.Ber;
import com.sun.jndi.ldap.BerDecoder;
import com.sun.jndi.ldap.BerEncoder;
import com.sun.jndi.ldap.ctl.DirSyncResponseControl;
import org.junit.Before;
import org.junit.Test;

import javax.naming.ldap.Control;
import javax.naming.ldap.LdapContext;
import javax.naming.ldap.SortControl;
import javax.naming.ldap.SortResponseControl;
import java.io.IOException;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

/**
 * Unit tests for the SortControlDirContextProcessor class.
 * 
 * @author Ulrik Sandberg
 */
public class SortControlDirContextProcessorTest {

    private LdapContext ldapContextMock;

	private SortControlDirContextProcessor tested;

    @Before
    public void setUp() throws Exception {
        tested = new SortControlDirContextProcessor("key");

        // Create ldapContext mock
        ldapContextMock = mock(LdapContext.class);
    }

    @Test
    public void testCreateRequestControl() throws Exception {
        SortControl result = (SortControl) tested.createRequestControl();
        assertNotNull(result);
        assertEquals("1.2.840.113556.1.4.473", result.getID());
        assertEquals(9, result.getEncodedValue().length);
    }

    @Test
    public void testPostProcess() throws Exception {
        byte sortResult = 0; // success

        byte[] value = encodeValue(sortResult);
        SortResponseControl control = new SortResponseControl(
                "dummy", true, value);

        when(ldapContextMock.getResponseControls()).thenReturn( new Control[]{control});

        tested.postProcess(ldapContextMock);

        assertEquals(true, tested.isSorted());
        assertEquals(0, tested.getResultCode());
    }

    @Test
    public void testPostProcess_NonSuccess() throws Exception {
        byte sortResult = 1;

        byte[] value = encodeValue(sortResult);
        SortResponseControl control = new SortResponseControl(
                "dummy", true, value);

        when(ldapContextMock.getResponseControls()).thenReturn(new Control[] { control });

        tested.postProcess(ldapContextMock);

        assertEquals(false, tested.isSorted());
        assertEquals(1, tested.getResultCode());
    }

    @Test
    public void testPostProcess_InvalidResponseControl() throws Exception {
        int resultSize = 50;
        byte pageSize = 8;

        byte[] value = new byte[1];
        value[0] = pageSize;
        byte[] cookie = encodeDirSyncValue(resultSize, value);

        // Using another response control to verify that it is ignored
        DirSyncResponseControl control = new DirSyncResponseControl("dummy",
                true, cookie);

        when(ldapContextMock.getResponseControls()).thenReturn(new Control[] { control });

        tested.postProcess(ldapContextMock);

        assertEquals(false, tested.isSorted());
    }

    @Test
    public void testBerDecoding() throws Exception {
        int sortResult = 53; // unwilling to perform
        byte[] encoded = encodeValue(sortResult);

        BerDecoder ber = new BerDecoder(encoded, 0, encoded.length);

        ber.parseSeq(null);
        int actualSortResult = ber.parseEnumeration();

        assertEquals("sortResult,", 53, actualSortResult);
    }

    private byte[] encodeValue(int sortResult) throws IOException {

        // build the ASN.1 encoding
        BerEncoder ber = new BerEncoder(10);

        ber.beginSeq(Ber.ASN_SEQUENCE | Ber.ASN_CONSTRUCTOR);
        ber.encodeInt(sortResult, Ber.ASN_ENUMERATED);
        ber.endSeq();

        return ber.getTrimmedBuf();
    }

    /**
     * Encode a value suitable for the DirSyncResponseControl used in a test.
     */
    private byte[] encodeDirSyncValue(int pageSize, byte[] cookie)
            throws IOException {

        // build the ASN.1 encoding
        BerEncoder ber = new BerEncoder(10 + cookie.length);

        ber.beginSeq(Ber.ASN_SEQUENCE | Ber.ASN_CONSTRUCTOR);
        ber.encodeInt(1); // flag
        ber.encodeInt(pageSize); // maxReturnLength
        ber.encodeOctetString(cookie, Ber.ASN_OCTET_STR);
        ber.endSeq();

        return ber.getTrimmedBuf();
    }
}
